package analyzer

import "fmt"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *Tree) analyzeExpression (
	into       entity.Type,
	mode       strictness,
	expression entity.Expression,
) (
	entity.Expression,
	error,
) {
	switch expression.(type) {
	case *entity.Variable:
		return this.analyzeVariable(into, mode, expression.(*entity.Variable))
	case *entity.Declaration:
		return this.analyzeDeclaration(into, mode, expression.(*entity.Declaration))
	case *entity.Call:
		return this.analyzeCall(into, mode, expression.(*entity.Call))
	case *entity.MethodCall:
		return this.analyzeMethodCall(into, mode, expression.(*entity.MethodCall))
	case *entity.Subscript:
		return this.analyzeSubscript(into, mode, expression.(*entity.Subscript))
	case *entity.Slice:
		return this.analyzeSlice(into, mode, expression.(*entity.Slice))
	case *entity.Dereference:
		return this.analyzeDereference(into, mode, expression.(*entity.Dereference))
	case *entity.Reference:
		return this.analyzeReference(into, mode, expression.(*entity.Reference))
	case *entity.ValueCast:
		return this.analyzeValueCast(into, mode, expression.(*entity.ValueCast))
	case *entity.BitCast:
		return this.analyzeBitCast(into, mode, expression.(*entity.BitCast))
	case *entity.Operation:
		return this.analyzeOperation(into, mode, expression.(*entity.Operation))
	case *entity.Block:
		return this.analyzeBlock(into, mode, expression.(*entity.Block))
	case *entity.MemberAccess:
		return this.analyzeMemberAccess(into, mode, expression.(*entity.MemberAccess))
	case *entity.IfElse:
		return this.analyzeIfElse(into, mode, expression.(*entity.IfElse))
	case *entity.Loop:
		return this.analyzeLoop(into, mode, expression.(*entity.Loop))
	case *entity.Break:
		return this.analyzeBreak(into, mode, expression.(*entity.Break))
	case *entity.Return:
		return this.analyzeReturn(into, mode, expression.(*entity.Return))

	case *entity.LiteralInt:
		return this.analyzeLiteralInt(into, mode, expression.(*entity.LiteralInt))
	case *entity.LiteralFloat:
		return this.analyzeLiteralFloat(into, mode, expression.(*entity.LiteralFloat))
	case *entity.LiteralArray:
		return this.analyzeLiteralArray(into, mode, expression.(*entity.LiteralArray))
	case *entity.LiteralStruct:
		return this.analyzeLiteralStruct(into, mode, expression.(*entity.LiteralStruct))
	case *entity.LiteralBoolean:
		return this.analyzeLiteralBoolean(into, mode, expression.(*entity.LiteralBoolean))
	default:
		panic(fmt.Sprint (
			"BUG: analyzer doesnt know about expression ",
			expression))
	}
}

func (this *Tree) analyzeStatement (
	statement entity.Statement,
) (
	entity.Statement,
	error,
) {
	if assignment, ok := statement.(*entity.Assignment); ok {
		return this.analyzeAssignment(assignment)
	} else if expression, ok := statement.(entity.Expression); ok {
		expression, err := this.analyzeExpression(nil, strict, expression)
		if err != nil { return nil, err }
		return expression.(entity.Statement), nil
	} else {
		panic(fmt.Sprint (
			"BUG: analyzer doesnt know about statement ",
			statement))
	}
}
